@@ -36,11 +36,21 @@
 **  RCS:
 **
 **	$Source: /afs/andrew.cmu.edu/usr18/dave64/work/IMAP_Proxy/src/RCS/main.c,v $
-**	$Id: main.c,v 1.31 2006/10/03 12:13:09 dave64 Exp $
+**	$Id: main.c,v 1.34 2007/11/15 11:14:16 dave64 Exp $
 **      
 **  Modification History:
 **
 **	$Log: main.c,v $
+**	Revision 1.34  2007/11/15 11:14:16  dave64
+**	Patch by Jose Luis Tallón to add pidfile support and
+**	enhance daemon behavior.
+**
+**	Revision 1.33  2007/05/31 12:10:59  dave64
+**	Applied ipv6 patch by Antonio Querubin.
+**
+**	Revision 1.32  2007/05/31 11:46:42  dave64
+**	Applied OpenSSL threads patch by Jan Grant.
+**
 **	Revision 1.31  2006/10/03 12:13:09  dave64
 **	Patch by Matt Selsky to log ssl peer verify at debug level instead
 **	of err level.
@@ -171,7 +181,7 @@
 */
 
 
-static char *rcsId = "$Id: main.c,v 1.31 2006/10/03 12:13:09 dave64 Exp $";
+static char *rcsId = "$Id: main.c,v 1.34 2007/11/15 11:14:16 dave64 Exp $";
 static char *rcsSource = "$Source: /afs/andrew.cmu.edu/usr18/dave64/work/IMAP_Proxy/src/RCS/main.c,v $";
 static char *rcsAuthor = "$Author: dave64 $";
 
@@ -256,37 +266,41 @@ static void SetBannerAndCapability( void );
 static int ParseBannerAndCapability( char *, unsigned int,
 				      char *, unsigned int );
 static void ServerInit( void );
+static void Daemonize( const char* );
 static void Usage( void );
 
 
 
 int main( int argc, char *argv[] )
 {
-    char *fn = "main()";
+    const char *fn = "main()";
     char f_randfile[ PATH_MAX ];
     int listensd;                      /* socket descriptor we'll bind to */
     int clientsd;                      /* incoming socket descriptor */
-    int addrlen;                       
-    struct sockaddr_in srvaddr;
-    struct sockaddr_in cliaddr;
+    int sockaddrlen;                       
+    struct sockaddr_storage srvaddr;
+    struct sockaddr_storage cliaddr;
     pthread_t ThreadId;                /* thread id of each incoming conn */
     pthread_t RecycleThread;           /* used just for the recycle thread */
     pthread_attr_t attr;               /* generic thread attribute struct */
     int rc, i, fd;
     unsigned int ui;
-    pid_t pid;                         /* used just for a fork call */
     struct linger lingerstruct;        /* for the socket reuse stuff */
     int flag;                          /* for the socket reuse stuff */
     ICC_Struct *ICC_tptr;             
     extern char *optarg;
     extern int optind;
     char ConfigFile[ MAXPATHLEN ];     /* path to our config file */
+    char PidFile[ MAXPATHLEN ];		/* path to our pidfile */
 #ifdef HAVE_LIBWRAP
     struct request_info r;             /* request struct for libwrap */
 #endif
+    struct addrinfo aihints, *ai;
+    int gaierrnum;
 
     flag = 1;
     ConfigFile[0] = '\0';
+    strncpy( PidFile, DEFAULT_PID_FILE, sizeof PidFile -1 );
 
     /*
      * Ignore signals we don't want to die from but we don't care enough
@@ -296,7 +310,7 @@ int main( int argc, char *argv[] )
     signal( SIGHUP, SIG_IGN );
     
 
-    while (( i = getopt( argc, argv, "f:h" ) ) != EOF )
+    while (( i = getopt( argc, argv, "f:p:h" ) ) != EOF )
     {
 	switch( i )
 	{
@@ -307,7 +321,15 @@ int main( int argc, char *argv[] )
 	    syslog( LOG_INFO, "%s: Using configuration file '%s'",
 		    fn, ConfigFile );
 	    break;
-	    
+        
+        case 'p':
+            /* user specified a pidfile */
+            strncpy( PidFile, optarg, sizeof PidFile -1 );
+            PidFile[ sizeof PidFile - 1 ] = '\0';
+            syslog( LOG_INFO, "%s: Using pidfile '%s'",
+            fn, PidFile );
+            break;
+        
 	case 'h':
 	    Usage();
 	    exit( 0 );
@@ -411,43 +433,8 @@ int main( int argc, char *argv[] )
     memset( ICC_HashTable, 0, sizeof ICC_HashTable );
 
     ServerInit();
-
-    /* detach from our parent if necessary */
-    if (! (getppid() == 1) && ( ! PC_Struct.foreground_mode ) )
-    {
-	syslog( LOG_INFO, "%s: Configured to run in background mode.", fn );
-	
-	if ( (pid = fork()) < 0)
-	{
-	    syslog(LOG_ERR, "%s: initial call to fork() failed: %s", fn, strerror(errno));
-	    exit( 1 );
-	}
-	else if ( pid > 0)
-	{
-	    exit( 0 );
-	}
-	
-	if (setsid() == -1)
-	{
-	    syslog(LOG_WARNING, "%s: setsid() failed: %s", 
-		   fn, strerror(errno));
-	}
-	if ( (pid = fork()) < 0)
-	{
-	    syslog(LOG_ERR, "%s: secondary call to fork() failed: %s", fn, 
-		   strerror(errno));
-	    exit( 1 );
-	}
-	else if ( pid > 0)
-	{
-	    exit( 0 );
-	}
-    }
-    else
-    {
-	syslog( LOG_INFO, "%s: Configured to run in foreground mode.", fn );
-    }
     
+    /* Daemonize() would go here */
 
     SetBannerAndCapability();
     
@@ -459,6 +446,9 @@ int main( int argc, char *argv[] )
 	{
 	    /* Initialize SSL_CTX */
 	    SSL_library_init();
+
+	    /* Set up OpenSSL thread protection */
+	    ssl_thread_setup(fn);
 	    
             /* Need to seed PRNG, too! */
             if ( RAND_egd( ( RAND_file_name( f_randfile, sizeof( f_randfile ) ) == f_randfile ) ? f_randfile : "/.rnd" ) ) 
@@ -507,32 +497,30 @@ int main( int argc, char *argv[] )
 	}
     }
     
-    memset( (char *) &srvaddr, 0, sizeof srvaddr );
-    srvaddr.sin_family = PF_INET;
-    if ( !PC_Struct.listen_addr )
-    {
-	srvaddr.sin_addr.s_addr = htonl(INADDR_ANY);
-    }
-    else 
-    {
-	srvaddr.sin_addr.s_addr = inet_addr( PC_Struct.listen_addr ); 
-	if ( srvaddr.sin_addr.s_addr  == -1 )
+    memset( &aihints, 0, sizeof aihints );
+    aihints.ai_family = AF_UNSPEC;
+    aihints.ai_socktype = SOCK_STREAM;
+    aihints.ai_flags = AI_PASSIVE;
+
+    if ( ( gaierrnum = getaddrinfo( PC_Struct.listen_addr,
+				    PC_Struct.listen_port,
+				    &aihints, &ai ) ) )
 	{
 	    syslog( LOG_ERR, "%s: bad bind address: '%s' specified in config file.  Exiting.", fn, PC_Struct.listen_addr );
 	    exit( 1 );
 	}
-    }
-    
 
-    syslog(LOG_INFO, "%s: Binding to tcp %s:%d", fn, PC_Struct.listen_addr ?
-	   PC_Struct.listen_addr : "*", PC_Struct.listen_port );
-    srvaddr.sin_port = htons(PC_Struct.listen_port);
+    syslog( LOG_INFO, "%s: Binding to tcp %s:%s", fn,
+	    PC_Struct.listen_addr ? PC_Struct.listen_addr : "*",
+	    PC_Struct.listen_port );
 
-    listensd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
+    for ( ; ai != NULL; ai = ai->ai_next )
+    {
+	listensd = socket( ai->ai_family, ai->ai_socktype, ai->ai_protocol );
     if ( listensd == -1 )
     {
-	syslog(LOG_ERR, "%s: socket() failed: %s", fn, strerror(errno));
-	exit( 1 );
+	    syslog(LOG_WARNING, "%s: socket() failed: %s", fn, strerror(errno));
+	    continue;
     }
 
     setsockopt(listensd, SOL_SOCKET, SO_REUSEADDR, (void *)&flag, 
@@ -549,11 +537,18 @@ int main( int argc, char *argv[] )
 	setsockopt( listensd, SOL_SOCKET, SO_KEEPALIVE, (void *)&lingerstruct.l_onoff, sizeof lingerstruct.l_onoff );
     }
 
-	    
-    if ( bind(listensd, (struct sockaddr *)&srvaddr, sizeof( srvaddr ) ) < 0 )
+	memcpy( &srvaddr, ai->ai_addr, ai->ai_addrlen );
+	if ( bind( listensd, (struct sockaddr *)&srvaddr, ai->ai_addrlen ) < 0 )
     {
-	syslog(LOG_ERR, "%s: bind() failed: %s", fn, strerror(errno) );
-	exit( 1 );
+	    syslog(LOG_WARNING, "%s: bind() failed: %s", fn, strerror(errno) );
+	    continue;
+	}
+	else break;
+    }
+    if ( ai == NULL )
+    {
+	syslog( LOG_ERR, "%s: no useable addresses to bind to", fn );
+	exit( EXIT_FAILURE);
     }
 
     /*
@@ -594,6 +589,12 @@ int main( int argc, char *argv[] )
     IMAPCount->StartTime = time( 0 );
     IMAPCount->CountTime = time( 0 );
 
+    /*
+     * Daemonize as late as possible, so that connection failures can be caught
+     * and startup aborted before dettaching from parent
+     */
+    Daemonize( PidFile );
+
     if ( BecomeNonRoot() )
 	exit( 1 );
 
@@ -638,10 +639,11 @@ int main( int argc, char *argv[] )
     {
 	/*
 	 * Bug fixed by Gary Mills <mills@cc.UManitoba.CA>.  I forgot
-	 * to initialize addrlen.
+	 * to initialize sockaddrlen.
 	 */
-	addrlen = sizeof cliaddr;
-	clientsd = accept( listensd, (struct sockaddr *)&cliaddr, &addrlen );
+	sockaddrlen = sizeof cliaddr;
+	clientsd = accept( listensd, (struct sockaddr *)&cliaddr,
+			   &sockaddrlen );
 	if ( clientsd == -1 )
 	{
 	    syslog(LOG_WARNING, "%s: accept() failed: %s -- retrying", 
@@ -701,7 +703,7 @@ int main( int argc, char *argv[] )
  */
 void Usage( void )
 {
-    printf("Usage: %s [-f config filename] [-h]\n", PGM );
+    printf("Usage: %s [-f config filename] [-p pidfile] [-h]\n", PGM );
     return;
 }
 
@@ -724,10 +726,11 @@ void Usage( void )
 static void ServerInit( void ) 
 {
     char *fn = "ServerInit()";
-    struct hostent *hp;
     struct rlimit rl;
     int rc;
     struct passwd *pw;
+    struct addrinfo aihints, *ai;
+    int gaierrnum, sd;
 
     
     /* open the global trace file and make proc_username own it */
@@ -780,13 +783,19 @@ static void ServerInit( void )
     syslog( LOG_INFO, "%s: proxying to IMAP server '%s'.", fn, 
 	    PC_Struct.server_hostname );
     
+    memset( &aihints, 0, sizeof aihints );
+    aihints.ai_family = AF_UNSPEC;
+    aihints.ai_socktype = SOCK_STREAM;
+
     for( ;; )
     {
-	hp = gethostbyname( PC_Struct.server_hostname );
-	
-	if ( !hp )
+	if ( ( gaierrnum = getaddrinfo( PC_Struct.server_hostname,
+					PC_Struct.server_port,
+					&aihints, &ai ) ) )
 	{
-	    syslog(LOG_ERR, "%s: gethostbyname() failed to resolve hostname of remote IMAP server: %s -- retrying", fn, strerror(errno) );
+	    syslog( LOG_ERR,
+		    "%s: getaddrinfo() failed to resolve hostname of remote IMAP server: %s -- retrying",
+		    fn, gai_strerror( gaierrnum ) );
 	    sleep( 15 );
 	}
 	else
@@ -795,23 +804,121 @@ static void ServerInit( void )
 	}
     }
 
-    memcpy( &ISD.host, hp, sizeof(struct hostent) );
-    
-    syslog(LOG_INFO, "%s: Proxying to IMAP port %d", 
+    syslog(LOG_INFO, "%s: Proxying to IMAP port %s", 
 	   fn, PC_Struct.server_port );
-    ISD.serv.s_port = htons(PC_Struct.server_port);
         
     /* 
      * fill in the address family, the host address, and the
      * service port of our global socket address structure
      */
-    ISD.srv.sin_family = PF_INET;
-    memcpy( &ISD.srv.sin_addr.s_addr, ISD.host.h_addr, ISD.host.h_length );
-    ISD.srv.sin_port = ISD.serv.s_port;
+    ISD.airesults = ai;
+    ISD.srv = NULL;
+    for ( ; ai != NULL; ai = ai->ai_next )
+    {
+        if ( ( sd = socket( ai->ai_family, ai->ai_socktype, ai->ai_protocol ) )
+	     < 0 ) continue;
+	if ( connect( sd, (struct sockaddr *)ai->ai_addr, ai->ai_addrlen ) )
+		continue;
+	close( sd );
+	ISD.srv = ai;
+	break;
+    }
+    if ( ai == NULL )
+    {
+	syslog( LOG_ERR, "%s: %s -- exiting", fn, strerror( errno ) );
+	exit( EXIT_FAILURE );
+    }
 }
 
 
+/*++
+ * Function:   Daemonize
+ *
+ * Purpose:    Daemonize, closing all unneeded descriptors.
+ *             Write the daemon's PID into 'pidfile'
+ *
+ * Parameters: pidfile -- where to write our PID.
+ *
+ * Returns:    nada -- exits on error
+ *
+ * Authors:    Jose Luis Tallon <jltallon@adv-solutions.net>
+ *
+ * Notes:
+ *--
+ */
+void Daemonize( const char* pidfile )
+{
+ const char* fn="Daemonize()";
+ FILE* fp=NULL;
+ pid_t pid;                         /* used just for a fork call */
+ int i;
 
+    /* detach from our parent if necessary */
+    if (! (getppid() == 1) && ( ! PC_Struct.foreground_mode ) )
+    {
+	syslog( LOG_INFO, "%s: Configured to run in background mode.", fn );
+
+	if ( (pid = fork()) < 0)
+	{
+		syslog(LOG_ERR, "%s: initial call to fork() failed: %s",
+			fn, strerror(errno));
+		exit( 1 );
+	}
+	else if ( pid > 0)
+	{
+		exit( 0 );
+	}
+
+	if (setsid() == -1)
+	{
+		syslog(LOG_WARNING, "%s: setsid() failed: %s",
+			fn, strerror(errno));
+	}
+	if ( (pid = fork()) < 0)
+	{
+		syslog(LOG_ERR, "%s: secondary call to fork() failed: %s",
+			fn, strerror(errno));
+		exit( 1 );
+	}
+	else if ( pid > 0)
+	{
+		exit( 0 );
+	}
+	if ( (fp=fopen(pidfile,"wt")) == NULL )
+	{
+		syslog(LOG_ERR, "%s: creating pidfile '%s' failed: %s", fn,
+			pidfile, strerror(errno));
+		exit(1);
+	}
+	if( fprintf(fp, "%u\n", (unsigned)getpid()) < 0 )
+	{
+		syslog(LOG_ERR, "%s: fprintf on pidfile failed: %s", fn,
+			strerror(errno));
+		exit(1);
+	}
+	fclose(fp);
+	if( chdir("/") < 0 )
+	{
+		syslog(LOG_ERR, "%s: chdir(\"/\") failed: %s", fn,
+			strerror(errno));
+		exit( 1 );
+	}
+	if( (i=open("/dev/null",O_RDWR)) < 0 )
+	{
+		syslog(LOG_ERR, "%s: open(\"/dev/null\") failed: %s", fn,
+		strerror(errno));
+		exit( 1 );
+	}
+	close(2); dup(i);
+	close(1); dup(i);
+	close(0); dup(i);
+	close(i);
+    }
+    else
+    {
+	syslog( LOG_INFO, "%s: Configured to run in foreground mode.", fn );
+    }
+}
 
 /*++
  * Function:	ParseBannerAndCapability
@@ -975,7 +1082,8 @@ static void SetBannerAndCapability( void )
 
     for ( ;; )
     {
-	sd = socket( PF_INET, SOCK_STREAM, IPPROTO_TCP );
+	sd = socket( ISD.srv->ai_family, ISD.srv->ai_socktype,
+		     ISD.srv->ai_protocol );
 	if ( sd == -1 )
 	{
 	    syslog(LOG_ERR, "%s: socket() failed: %s -- exiting", fn, 
@@ -983,9 +1091,8 @@ static void SetBannerAndCapability( void )
 	    exit( 1 );
 	}
 	
-	
-	if ( connect( sd, (struct sockaddr *)&ISD.srv, sizeof(ISD.srv) ) == -1 )
-	{
+	if ( connect( sd, (struct sockaddr *)ISD.srv->ai_addr, 
+		      ISD.srv->ai_addrlen ) == -1 ) 	{
 	    syslog(LOG_ERR, "%s: connect() to imap server on socket [%d] failed: %s -- retrying", fn, sd, strerror(errno));
 	    close( sd );
 	    
